#!/usr/bin/env python
#
# Author: Thamme Gowda [tg\tat isi\tdot edu] 
# Created: 5/5/20
data = """ace\tAchinese
af\tAfrikaans
ak\tAkan
sq\tAlbanian
am\tAmharic
ar_TN\tArabic
ar_SY\tArabic
ara\tArabic
ar\tArabic
an\tAragonese
hy\tArmenian
as\tAssamese
ast\tAsturian
aym\tAymara
az_IR\tAzerbaijani
az\tAzerbaijani
bal\tBaluchi
ba\tBashkir
eu\tBasque
be\tBelarusian
bem\tBemba
bn\tBengali
bn_IN\tBengali
ber\tBerber languages
bho\tBhojpuri
byn\tBlin
nb\tBokmål, Norwegian
nb_NO\tBokmål, Norwegian
bs\tBosnian
br\tBreton
bg_BG\tBulgarian
bg\tBulgarian
bua\tBuriat
my\tBurmese
cat\tCatalan
ca\tCatalan
ceb\tCebuano
km\tCentral Khmer
ce\tChechen
chr\tCherokee
ny\tChichewa
zh\tChinese
zh_HK\tChinese
zh_CN\tChinese
zh_TW\tChinese
cv\tChuvash
kw\tCornish
co\tCorsican
mus\tCreek
crh\tCrimean Tatar
hr\tCroatian
cs\tCzech
da\tDanish
da_DK\tDanish
dv\tDivehi
nl_NL\tDutch
nl\tDutch
dz\tDzongkha
en_ZA\tEnglish
en\tEnglish
en_AU\tEnglish
en_US\tEnglish
en_CA\tEnglish
en_GB\tEnglish
en_NZ\tEnglish
ang\tEnglish, Old 
eo\tEsperanto
et\tEstonian
fo\tFaroese
fil\tFilipino
fi\tFinnish
fr\tFrench
fr_CA\tFrench
fr_FR\tFrench
frm\tFrench, Middle 
fur\tFriulian
ff\tFulah
gd\tGaelic
gl\tGalician
lg\tGanda
ka\tGeorgian
de_AT\tGerman
de\tGerman
de_DE\tGerman
de_CH\tGerman
el\tGreek
grc\tGreek, Ancient 
gn\tGuarani
gu\tGujarati
ht\tHaitian
ha\tHausa
haw\tHawaiian
he\tHebrew
hil\tHiligaynon
hi\tHindi
hi_IN\tHindi
hu\tHungarian
is\tIcelandic
io\tIdo
ig\tIgbo
id\tIndonesian
ia\tInterlingua 
iu\tInuktitut
ga\tIrish
it_IT\tItalian
it\tItalian
ja\tJapanese
jv\tJavanese
kab\tKabyle
kl\tKalaallisut
xal\tKalmyk
kn\tKannada
kr\tKanuri
ks\tKashmiri
csb\tKashubian
kk\tKazakh
rw\tKinyarwanda
ky\tKirghiz
tlh\tKlingon
kg\tKongo
kok\tKonkani
ko\tKorean
ku\tKurdish
lo\tLao
la\tLatin
lv\tLatvian
li\tLimburgan
ln\tLingala
lt\tLithuanian
jbo\tLojban
nds\tLow German
dsb\tLower Sorbian
lb\tLuxembourgish
mk\tMacedonian
mai\tMaithili
mg\tMalagasy
ms\tMalay
ms_MY\tMalay
ml\tMalayalam
mt\tMaltese
gv\tManx
mi\tMaori
mr\tMarathi
mh\tMarshallese
mn\tMongolian
nqo\tN'Ko
nr\tNdebele, South
nap\tNeapolitan
ne\tNepali
se\tNorthern Sami
no_nb\tNorwegian
no\tNorwegian
nn_NO\tNorwegian Nynorsk
nn\tNorwegian Nynorsk
oc\tOccitan 
oj\tOjibwa
or\tOriya
om\tOromo
os\tOssetian
pam\tPampanga
pa\tPanjabi
pap\tPapiamento
nso\tPedi
fa\tPersian
fa_AF\tPersian
fa_IR\tPersian
pl\tPolish
pt_PT\tPortuguese
pt_BR\tPortuguese
pt\tPortuguese
ps\tPushto
qu\tQuechua
ro\tRomanian
rm\tRomansh
rom\tRomany
ru\tRussian
sm\tSamoan
sa\tSanskrit
sc\tSardinian
sco\tScots
sr_ME\tSerbian
sr\tSerbian
shn\tShan
sn\tShona
sd\tSindhi
si\tSinhala
sk\tSlovak
sl\tSlovenian
so\tSomali
son\tSonghai languages
st\tSotho, Southern
es_PE\tSpanish
es_CO\tSpanish
es_DO\tSpanish
es_HN\tSpanish
es_MX\tSpanish
es_ES\tSpanish
es_SV\tSpanish
es_UY\tSpanish
es_EC\tSpanish
es_PR\tSpanish
es_NI\tSpanish
es_VE\tSpanish
es\tSpanish
es_PA\tSpanish
es_AR\tSpanish
es_CR\tSpanish
es_CL\tSpanish
es_GT\tSpanish
sw\tSwahili
sv\tSwedish
syr\tSyriac
tl\tTagalog
tl_PH\tTagalog
tg_TJ\tTajik
tg\tTajik
ta_LK\tTamil
ta\tTamil
tt\tTatar
te\tTelugu
tet\tTetum
th\tThai
bo\tTibetan
ti\tTigrinya
ts\tTsonga
tr\tTurkish
tr_TR\tTurkish
tk\tTurkmen
tw\tTwi
ug\tUighur
uk\tUkrainian
hsb\tUpper Sorbian
ur\tUrdu
ur_PK\tUrdu
uz\tUzbek
ve\tVenda
vi\tVietnamese
vi_VN\tVietnamese
wa\tWalloon
cy\tWelsh
fy\tWestern Frisian
wo\tWolof
xh\tXhosa
yi\tYiddish
yo\tYoruba
zza\tZaza
zu\tZulu
ary\tary
brx\tbrx
ckb\tckb
foo\tfoo
frp\tfrp
gr\tgr
guc\tguc
hne\thne
jp\tjp
ksh\tksh
lij\tlij
lld\tlld
ltg\tltg
mhr\tmhr
miq\tmiq
mo\tmo
nan\tnan
nhn\tnhn
pms\tpms
pmy\tpmy
quz\tquz
sh\tsh
shs\tshs
sml\tsml
szl\tszl
tmp\ttmp
trv\ttrv
vec\tvec
wae\twae
zhs\tzhs
zht\tzht"""

def _opus_to_iso3_map():
    from mtdata.iso.iso639_3 import name_to_code, code_to_name
    from mtdata.iso.iso639_2 import code2_to_code3_name, CODE2_TO_3 as code2_to_3
    mapping = {}
    for line in data.splitlines():
        code, name = line.split('\t')
        iso_name, iso_code = '', ''
        if len(code) == 3 and code_to_name(code, None):
            iso_name = code_to_name(code)
            iso_code = name_to_code(iso_name)
        elif len(code) == 2 and code in code2_to_3:
            iso_code, iso_name = code2_to_code3_name(code)
            iso_code = name_to_code(iso_name) or iso_code
            assert iso_code
        elif name_to_code(name):
            iso_code = name_to_code(name)
        if iso_code and '_' not in code:
            mapping[code] = iso_code
    return mapping


opus_to_iso3 = _opus_to_iso3_map()

if __name__ == '__main__':
    from mtdata.iso.iso639_3 import code_to_name as iso3_code_to_name
    for code, iso_code in opus_to_iso3.items():
        print(f'{code}\t{iso_code}\t{iso3_code_to_name(iso_code)}')



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 5/8/20
import pytest

from mtdata.iso import iso3_code

def test_iso3_code():
    assert iso3_code("kn") == 'kan'
    assert iso3_code("KN") == 'kan'
    assert iso3_code("Kannada") == 'kan'
    assert iso3_code("kannada") == 'kan'
    assert iso3_code("kan") == 'kan'
    assert iso3_code("KANNADA") == 'kan'
    assert iso3_code("KaNnAdA") == 'kan'
    assert iso3_code("KAN") == 'kan'

    assert iso3_code("ne") == 'nep'
    assert iso3_code("nep") == 'nep'
    assert iso3_code("Nepali") == 'nep'
    assert iso3_code("Nepali (individual)") == 'npi'

    assert iso3_code("xyz") == None
    assert iso3_code("xyz", default="Error") == "Error"
    try:
        iso3_code("xyz", fail_error=True)
        assert False, 'Expected an exception'
    except:
        assert True




#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu]
# Created: 4/7/20

import scrapy

from mtdatabot.items import MTDataItem, MTCorpusItem

class OpusSpider(scrapy.Spider):
    name = 'tilde'
    start_urls = ['https://tilde-model.s3-eu-west-1.amazonaws.com/Tilde_MODEL_Corpus.html']

    def parse(self, response):
        suffix = '.tmx.zip'
        anchors = response.xpath(f"//a[contains(@href, '{suffix}')]")
        self.logger.info(f"Found {len(anchors)} pairs")
        for anch in anchors:
            url = response.urljoin(anch.xpath('./@href').get())
            file_path = url.split('/')[-1]
            parts = file_path.replace(suffix, '').split('.')
            lang_pair = parts[-1].split('-')
            name = parts[0]
            if len(parts) != 2 or len(lang_pair) != 2:
                self.logger.warning(f"Couldnt parse {url} properly; skipped")
                continue

            item = MTDataItem()
            item['url'] = url
            item['name'] = name
            item['id'] = file_path
            item['langs'] = lang_pair
            item['title'] = anch.xpath('./@title').get().strip().replace(', TMX format', '')
            item['info'] = anch.xpath('./text()').get().strip()
            yield item


#!/usr/bin/env python
#
# parses the min jsonline file from OPUS
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 5/9/20
import json
import logging as log

log.basicConfig(level=log.INFO)


def main(args=None):
    args = args or parse_args()
    args.out.write('data="""')
    for line in args.inp:
        data = json.loads(line)
        url = data['url'].split("?f=")[-1]
        name = data['name']
        langs = " ".join(f"{l1}-{l2}" for l1, l2 in data['langs'])
        args.out.write(f"{name}\t{url}\t{langs}\n")
    args.out.write('"""')

def parse_args():
    import argparse
    import sys
    import io
    stdin = io.TextIOWrapper(sys.stdin, encoding='utf-8', errors='ignore')
    stdout = io.TextIOWrapper(sys.stdout, encoding='utf-8', errors='ignore')
    p = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    p.add_argument('-i', '--inp', type=argparse.FileType('r', encoding='utf-8'), default=stdin,
                   help='Input file path')
    p.add_argument('-o', '--out', type=argparse.FileType('w', encoding='utf-8'), default=stdout,
                   help='Output file path')
    return p.parse_args()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 5/8/20
import logging as log

log.basicConfig(level=log.INFO)


def main(langs=None):
    from mtdata.iso import iso3_code
    from mtdata.iso.iso639_3 import code_to_name, data as ISO639_3
    langs = langs or parse_args().get('langs', [])
    if not langs:
        print(ISO639_3)
    else:
        print(f"Input\tISO639_3\tName")
        for lang in langs:
            iso_code = iso3_code(lang)
            iso_name = code_to_name(iso_code) if iso_code else '-none-'
            iso_code = iso_code or '-none-'
            print(f"{lang}\t{iso_code}\t{iso_name}")


def parse_args():
    import argparse
    p = argparse.ArgumentParser(prog='python -m mtdata.iso', description="ISO 639-3 lookup tool")
    p.add_argument("langs", nargs='*', help="Language code or name that needs to be looked up. "
                                            "When no language code is given, all languages are listed.")
    return vars(p.parse_args())


if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 5/10/20
from mtdata.index import Index, Entry


def load_all(index: Index):
    data = """am-en am-fr ar-am ar-en ar-fr aym-am aym-ar aym-en aym-fr bg-ar bg-aym bg-en bg-fr
  bn-am bn-ar bn-aym bn-bg bn-en bn-fr ca-am ca-ar ca-aym ca-bg ca-bn ca-en ca-fr cs-ar cs-aym 
  cs-bg cs-bn cs-ca cs-en cs-fr da-am da-ar da-aym da-bg da-bn da-ca da-cs da-en da-fr de-am 
  de-ar de-aym de-bg de-bn de-ca de-cs de-da de-en de-fr el-am el-ar el-aym el-bg el-bn el-ca
  el-cs el-da el-de el-en el-fr eo-ar eo-aym eo-bg eo-bn eo-ca eo-cs eo-da eo-de eo-el eo-en
  eo-fr es-am es-ar es-aym es-bg es-bn es-ca es-cs es-da es-de es-el es-en es-eo es-fr fa-am
  fa-ar fa-aym fa-bg fa-bn fa-ca fa-cs fa-da fa-de fa-el fa-en fa-eo fa-es fa-fr fil-ar fil-aym
  fil-bg fil-bn fil-ca fil-cs fil-da fil-de fil-el fil-en fil-eo fil-es fil-fa fil-fr fr-en he-ar
   he-bn he-ca he-cs he-da he-de he-el he-en he-es he-fa he-fr hi-am hi-ar hi-bg hi-bn hi-cs hi-de
   hi-el hi-en hi-eo hi-es hi-fa hi-fr hu-am hu-ar hu-aym hu-bg hu-bn hu-ca hu-cs hu-da hu-de
   hu-el hu-en hu-eo hu-es hu-fa hu-fil hu-fr hu-hi id-am id-ar id-aym id-bg id-bn id-ca id-cs 
   id-da id-de id-el id-en id-eo id-es id-fa id-fil id-fr id-hi id-hu it-am it-ar it-aym it-bg it-bn 
   it-ca it-cs it-da it-de it-el it-en it-eo it-es it-fa it-fil it-fr it-he it-hi it-hu it-id jp-am 
   jp-ar jp-aym jp-bg jp-bn jp-ca jp-cs jp-da jp-de jp-el jp-en jp-eo jp-es jp-fa jp-fil jp-fr 
   jp-he jp-hi jp-hu jp-id jp-it km-ar km-aym km-bn km-ca km-da km-de km-el km-en km-es km-fa km-fil 
   km-fr km-hu km-it km-jp ko-am ko-ar ko-aym ko-bg ko-bn ko-ca ko-cs ko-da ko-de ko-el ko-en ko-eo 
   ko-es ko-fa ko-fil ko-fr ko-hu ko-id ko-it ko-jp ku-ar ku-el ku-en ku-es ku-fr ku-it ku-jp mg-am 
   mg-ar mg-aym mg-bg mg-bn mg-ca mg-cs mg-da mg-de mg-el mg-en mg-eo mg-es mg-fa mg-fil mg-fr 
   mg-he mg-hi mg-hu mg-id mg-it mg-jp mg-km mg-ko mg-ku mk-am mk-ar mk-aym mk-bg mk-bn mk-ca mk-cs 
   mk-da mk-de mk-el mk-en mk-eo mk-es mk-fa mk-fil mk-fr mk-he mk-hi mk-hu mk-id mk-it mk-jp mk-km 
   mk-ko mk-mg my-am my-ar my-aym my-bg my-bn my-ca my-cs my-da my-de my-el my-en my-es my-fa my-fil 
   my-fr my-he my-hi my-hu my-id my-it my-jp my-ko my-mg my-mk ne-ar ne-aym ne-bg ne-bn ne-ca ne-cs 
   ne-de ne-el ne-en ne-eo ne-es ne-fa ne-fr ne-hi ne-id ne-it ne-jp ne-ko ne-mg ne-mk nl-am nl-ar 
   nl-aym nl-bg nl-bn nl-ca nl-cs nl-da nl-de nl-el nl-en nl-eo nl-es nl-fa nl-fil nl-fr nl-he nl-hi 
   nl-hu nl-id nl-it nl-jp nl-km nl-ko nl-mg nl-mk nl-my nl-ne or-ar or-aym or-bn or-ca or-cs or-de 
   or-el or-en or-es or-fa or-fr or-hi or-it or-jp or-mg or-mk or-nl pa-ar pa-bn pa-ca pa-cs pa-de 
   pa-el pa-en pa-es pa-fr pa-hi pa-hu pa-it pa-jp pa-ko pa-mg pa-mk pa-ne pa-nl pl-am pl-ar pl-aym 
   pl-bg pl-bn pl-ca pl-cs pl-da pl-de pl-el pl-en pl-eo pl-es pl-fa pl-fil pl-fr pl-he pl-hi pl-hu 
   pl-id pl-it pl-jp pl-ko pl-ku pl-mg pl-mk pl-my pl-ne pl-nl pl-or pl-pa pt-am pt-ar pt-aym pt-bg 
   pt-bn pt-ca pt-cs pt-da pt-de pt-el pt-en pt-eo pt-es pt-fa pt-fil pt-fr pt-he pt-hi pt-hu pt-id 
   pt-it pt-jp pt-km pt-ko pt-ku pt-mg pt-mk pt-my pt-ne pt-nl pt-or pt-pa pt-pl ro-ar ro-aym ro-bg 
   ro-bn ro-ca ro-cs ro-de ro-el ro-en ro-eo ro-es ro-fa ro-fr ro-hu ro-id ro-it ro-jp ro-ko ro-ku 
   ro-mg ro-mk ro-my ro-ne ro-nl ro-pl ro-pt ru-am ru-ar ru-aym ru-bg ru-bn ru-ca ru-cs ru-da ru-de 
   ru-el ru-en ru-eo ru-es ru-fa ru-fil ru-fr ru-he ru-hi ru-hu ru-id ru-it ru-jp ru-km ru-ko ru-mg 
   ru-mk ru-my ru-ne ru-nl ru-or ru-pa ru-pl ru-pt ru-ro sq-am sq-ar sq-aym sq-bg sq-bn sq-ca sq-cs 
   sq-da sq-de sq-el sq-en sq-eo sq-es sq-fa sq-fil sq-fr sq-hi sq-hu sq-id sq-it sq-jp sq-ko sq-mg 
   sq-mk sq-my sq-nl sq-pl sq-pt sq-ru sr-am sr-ar sr-aym sr-bg sr-bn sr-ca sr-cs sr-da sr-de sr-el 
   sr-en sr-eo sr-es sr-fa sr-fil sr-fr sr-hi sr-hu sr-id sr-it sr-jp sr-km sr-ko sr-mg sr-mk sr-my 
   sr-ne sr-nl sr-pl sr-pt sr-ro sr-ru sr-sq sv-am sv-ar sv-aym sv-bg sv-bn sv-ca sv-cs sv-da sv-de 
   sv-el sv-en sv-eo sv-es sv-fa sv-fil sv-fr sv-he sv-hi sv-hu sv-id sv-it sv-jp sv-ko sv-mg sv-mk 
   sv-my sv-nl sv-pl sv-pt sv-ro sv-ru sv-sq sv-sr sw-am sw-ar sw-aym sw-bg sw-bn sw-ca sw-cs sw-da 
   sw-de sw-el sw-en sw-eo sw-es sw-fa sw-fil sw-fr sw-he sw-hi sw-hu sw-id sw-it sw-jp sw-km sw-ko 
   sw-mg sw-mk sw-my sw-ne sw-nl sw-pa sw-pl sw-pt sw-ro sw-ru sw-sq sw-sr sw-sv tet-ar tet-aym 
   tet-bn tet-cs tet-de tet-el tet-en tet-es tet-fr tet-id tet-it tet-mg tet-pt tet-ru tet-sw tr-am 
   tr-ar tr-aym tr-bg tr-bn tr-ca tr-cs tr-da tr-de tr-el tr-en tr-eo tr-es tr-fa tr-fil tr-fr tr-he 
   tr-hi tr-hu tr-id tr-it tr-jp tr-ko tr-mg tr-mk tr-my tr-ne tr-nl tr-pa tr-pl tr-pt tr-ro tr-ru 
   tr-sq tr-sr tr-sv tr-sw ur-am ur-ar ur-aym ur-bg ur-bn ur-ca ur-cs ur-da ur-de ur-el ur-en ur-eo 
   ur-es ur-fa ur-fil ur-fr ur-he ur-hi ur-hu ur-id ur-it ur-jp ur-ko ur-mg ur-mk ur-my ur-ne ur-nl 
   ur-or ur-pa ur-pl ur-pt ur-ro ur-ru ur-sq ur-sr ur-sv ur-sw ur-tr yo-ar yo-el yo-en yo-es yo-fr 
   yo-it yo-mg yo-pl yo-pt yo-ru yo-sw zhs-am zhs-ar zhs-aym zhs-bg zhs-bn zhs-ca zhs-cs zhs-da 
   zhs-de zhs-el zhs-en zhs-eo zhs-es zhs-fa zhs-fil zhs-fr zhs-he zhs-hi zhs-hu zhs-id zhs-it 
   zhs-jp zhs-km zhs-ko zhs-mg zhs-mk zhs-my zhs-ne zhs-nl zhs-pa zhs-pl zhs-pt zhs-ro zhs-ru 
   zhs-sq zhs-sr zhs-sv zhs-sw zhs-tr zhs-ur zht-am zht-ar zht-aym zht-bg zht-bn zht-ca zht-cs 
   zht-da zht-de zht-el zht-en zht-eo zht-es zht-fa zht-fil zht-fr zht-he zht-hi zht-hu zht-id 
   zht-it zht-jp zht-km zht-ko zht-mg zht-mk zht-my zht-ne zht-nl zht-pa zht-pl zht-pt zht-ro 
   zht-ru zht-sq zht-sr zht-sv zht-sw zht-tet zht-tr zht-ur zht-zhs"""
    url = 'http://casmacat.eu/corpus/global-voices-tar-balls/training.tgz'
    cite = """Philipp Koehn, "Global Voices Corpus" http://casmacat.eu/corpus/global-voices.html """

    # any hot fixes for lang id mapping specific to this source
    code_map = {
        'jp': 'jpn', # there was never a jp in ISO 693, it was always a 'ja' not 'jp'
        'zhs': 'zho'  # map simplified to chinese
     }
    code_map = code_map.get
    for pair in data.split():
        if 'zht' in pair:
            continue  #skipping traditional chinese because I dont know the ISO code for it
        l1, l2 = pair.split('-')
        f1 = f'training/globalvoices.{l1}-{l2}.{l1}'
        f2 = f'training/globalvoices.{l1}-{l2}.{l2}'
        l1, l2 = code_map(l1, l1), code_map(l2, l2)  # map codes
        ent = Entry(langs=(l1, l2), name='GlobalVoices_2018Q4', url=url,
                    filename='GlobalVoices_2018Q4-training.tgz', in_ext='txt', cite=cite,
                    in_paths=[f1, f2])
        index.add_entry(ent)



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 4/8/20
from mtdata.index import Entry, Index
import itertools


def load(index: Index):
    WMT13_CCRAWL = "http://www.statmt.org/wmt13/training-parallel-commoncrawl.tgz"
    WMT14_CITE = """@proceedings{ws-2014-statistical,
        title = "Proceedings of the Ninth Workshop on Statistical Machine Translation",
        editor = "Bojar, Ond{\v{r}}ej  and
          Buck, Christian  and
          Federmann, Christian  and
          Haddow, Barry  and
          Koehn, Philipp  and
          Monz, Christof  and
          Post, Matt  and
          Specia, Lucia",
        month = jun,
        year = "2014",
        address = "Baltimore, Maryland, USA",
        publisher = "Association for Computational Linguistics",
        url = "https://www.aclweb.org/anthology/W14-3300",
        doi = "10.3115/v1/W14-33",
    }"""
    for l1 in ['de', 'cs', 'fr', 'ru', 'es']:
        l2 = 'en'
        f1 = f'commoncrawl.{l1}-en.{l1}'
        f2 = f'commoncrawl.{l1}-en.en'
        index.add_entry(Entry(langs=(l1, l2), name=f'wmt13_commoncrawl', url=WMT13_CCRAWL,
                              filename='wmt13_parallel_commoncrawl.tgz',
                              in_paths=[f1, f2], in_ext='txt', cite=WMT14_CITE))

    # === WMT 13 release of europarl_v7 ===
    for l1 in ['cs', 'de', 'fr', 'es']:
        l2 = 'en'
        f1 = f'training/europarl-v7.{l1}-{l2}.{l1}'
        f2 = f'training/europarl-v7.{l1}-{l2}.{l2}'
        index.add_entry(Entry(langs=(l1, l2), name=f'wmt13_europarl_v7',
                              url="http://www.statmt.org/wmt13/training-parallel-europarl-v7.tgz",
                              filename="wmt13_europarl_v7.tgz",
                              in_paths=[f1, f2], in_ext='txt', cite=WMT14_CITE))

    # ==== WMT 18  news commentary v13 ===
    for l1 in ['cs', 'de', 'ru', 'zh']:
        l2 = 'en'
        f1 = f'training-parallel-nc-v13/news-commentary-v13.{l1}-{l2}.{l1}'
        f2 = f'training-parallel-nc-v13/news-commentary-v13.{l1}-{l2}.{l2}'
        index.add_entry(Entry(langs=(l1, l2), name=f'wmt18_news_commentary_v13',
                              url="http://data.statmt.org/wmt18/translation-task/training-parallel-nc-v13.tgz",
                              filename="wmt18_news_commentary_v13.tgz",
                              in_paths=[f1, f2], in_ext='txt', cite=WMT14_CITE))

    # === Europarl V9 corpus
    EUROPARL_v9 = 'http://www.statmt.org/europarl/v9/training/europarl-v9.%s-%s.tsv.gz'
    cite = r"""@inproceedings{koehn2005europarl,
      title={Europarl: A parallel corpus for statistical machine translation},
      author={Koehn, Philipp},
      booktitle={MT summit},
      volume={5},
      pages={79--86},
      year={2005},
      organization={Citeseer}
    }"""
    for pair in ['de en', 'cs en', 'cs pl', 'es pt', 'fi en', 'lt en']:
        l1, l2 = pair.split()
        index.add_entry(
            Entry(langs=(l1, l2), name='europarl_v9', url=EUROPARL_v9 % (l1, l2), cite=cite))

    # === Europarl V7 corpus
    EUROPARL_v7 = 'http://www.statmt.org/europarl/v7/%s-%s.tgz'
    cite = r"""@inproceedings{bojar-etal-2017-findings,
      title = "Findings of the 2017 Conference on Machine Translation ({WMT}17)",
      author = "Bojar, Ond{\v{r}}ej  and
        Chatterjee, Rajen  and
        Federmann, Christian  and
        Graham, Yvette  and
        Haddow, Barry  and
        Huang, Shujian  and
        Huck, Matthias  and
        Koehn, Philipp  and
        Liu, Qun  and
        Logacheva, Varvara  and
        Monz, Christof  and
        Negri, Matteo  and
        Post, Matt  and
        Rubino, Raphael  and
        Specia, Lucia  and
        Turchi, Marco",
      booktitle = "Proceedings of the Second Conference on Machine Translation",
      month = sep,
      year = "2017",
      address = "Copenhagen, Denmark",
      publisher = "Association for Computational Linguistics",
      url = "https://www.aclweb.org/anthology/W17-4717",
      doi = "10.18653/v1/W17-4717",
      pages = "169--214",
    }"""
    for l1 in 'bg cs da de el es et fi fr hu it lt lv nl pl pt ro sk sl sv'.split():
        l2 = 'en'
        src = f'europarl-v7.{l1}-{l2}.{l1}'
        ref = f'europarl-v7.{l1}-{l2}.{l2}'
        index.add_entry(Entry(langs=(l1, l2), name='europarl_v7', in_paths=[src, ref],
                              url=EUROPARL_v7 % (l1, l2), in_ext='txt', cite=cite))

    # === Digital Corpus of European Parliament
    index.add_entry(Entry(langs=('lv', 'en'), name='wmt17_dcep_v1',
                          in_paths=['*/*.lv', f'*/*.en'], cite=cite,
                          url='http://data.statmt.org/wmt17/translation-task/dcep.lv-en.v1.tgz'))
    index.add_entry(Entry(langs=('lv', 'en'), name='wmt17_books_v1',
                          in_paths=['*/*.lv', f'*/*.en'], cite=cite,
                          url='http://data.statmt.org/wmt17/translation-task/books.lv-en.v1.tgz'))

    # === News Commentary v14
    NEWSCOM_v14 = "http://data.statmt.org/news-commentary/v14/training/news-commentary-v14.%s-%s.tsv.gz"
    cite = r"""@inproceedings{bojar-etal-2018-findings,
        title = "Findings of the 2018 Conference on Machine Translation ({WMT}18)",
        author = "Bojar, Ond{\v{r}}ej  and
          Federmann, Christian  and
          Fishel, Mark  and
          Graham, Yvette  and
          Haddow, Barry  and
          Koehn, Philipp  and
          Monz, Christof",
        booktitle = "Proceedings of the Third Conference on Machine Translation: Shared Task Papers",
        month = oct,
        year = "2018",
        address = "Belgium, Brussels",
        publisher = "Association for Computational Linguistics",
        url = "https://www.aclweb.org/anthology/W18-6401",
        doi = "10.18653/v1/W18-6401",
        pages = "272--303"
    }"""
    for pair in ['ar cs', 'ar de', 'ar en', 'ar es', 'ar fr', 'ar hi', 'ar id', 'ar it',
                 'ar ja', 'ar kk', 'ar nl', 'ar pt', 'ar ru', 'ar zh', 'cs de', 'cs en', 'cs es',
                 'cs fr', 'cs hi', 'cs id', 'cs it', 'cs ja', 'cs kk', 'cs nl', 'cs pt', 'cs ru',
                 'cs zh', 'de en', 'de es', 'de fr', 'de hi', 'de id', 'de it', 'de ja', 'de kk',
                 'de nl', 'de pt', 'de ru', 'de zh', 'en es', 'en fr', 'en hi', 'en id', 'en it',
                 'en ja', 'en kk', 'en nl', 'en pt', 'en ru', 'en zh', 'es fr', 'es hi', 'es id',
                 'es it', 'es ja', 'es kk', 'es nl', 'es pt', 'es ru', 'es zh', 'fr hi', 'fr id',
                 'fr it', 'fr ja', 'fr kk', 'fr nl', 'fr pt', 'fr ru', 'fr zh', 'hi id', 'hi it',
                 'hi nl', 'hi pt', 'hi ru', 'hi zh', 'id it', 'id kk', 'id nl', 'id pt', 'id ru',
                 'id zh', 'it kk', 'it nl', 'it pt', 'it ru', 'it zh', 'ja ru', 'ja zh', 'kk nl',
                 'kk pt', 'kk ru', 'kk zh', 'nl pt', 'nl ru', 'nl zh', 'pt ru', 'pt zh', 'ru zh']:
        l1, l2 = pair.split()
        index.add_entry(
            Entry(langs=(l1, l2), name='news_commentary_v14', url=NEWSCOM_v14 % (l1, l2),
                  cite=cite))

    # ===== Wiki Titles V1
    WIKI_TITLES_v1 = 'http://data.statmt.org/wikititles/v1/wikititles-v1.%s-%s.tsv.gz'
    cite = r"""@inproceedings{barrault-etal-2019-findings,
        title = "Findings of the 2019 Conference on Machine Translation ({WMT}19)",
        author = {Barrault, Lo{\"\i}c  and
          Bojar, Ond{\v{r}}ej  and
          Costa-juss{\`a}, Marta R.  and
          Federmann, Christian  and
          Fishel, Mark  and
          Graham, Yvette  and
          Haddow, Barry  and
          Huck, Matthias  and
          Koehn, Philipp  and
          Malmasi, Shervin  and
          Monz, Christof  and
          M{\"u}ller, Mathias  and
          Pal, Santanu  and
          Post, Matt  and
          Zampieri, Marcos},
        booktitle = "Proceedings of the Fourth Conference on Machine Translation (Volume 2: Shared Task Papers, Day 1)",
        month = aug,
        year = "2019",
        address = "Florence, Italy",
        publisher = "Association for Computational Linguistics",
        url = "https://www.aclweb.org/anthology/W19-5301",
        doi = "10.18653/v1/W19-5301",
        pages = "1--61"
    }"""
    for pair in ['cs en', 'cs pl', 'de en', 'es pt', 'fi en', 'gu en', 'hi ne', 'kk en', 'lt en',
                 'ru en', 'zh en']:
        l1, l2 = pair.split()
        index.add_entry(Entry(langs=(l1, l2), name='wiki_titles_v1', url=WIKI_TITLES_v1 % (l1, l2),
                              cite=cite))

    # ===== Wiki Titles V2
    WIKI_TITLES_v2 = 'http://data.statmt.org/wikititles/v2/wikititles-v2.%s-%s.tsv.gz'
    for pair in ['ca es', 'cs en', 'de en', 'de fr', 'es pt', 'iu en', 'ja en', 'pl en', 'ps en',
                 'ru en', 'ta en', 'zh en']:
        l1, l2 = pair.split()
        index.add_entry(Entry(langs=(l1, l2), name='wiki_titles_v2', url=WIKI_TITLES_v2 % (l1, l2),
                              cite=cite))

    # ==== WMT  Dev and Tests
    wmt_sets = {
        'newstest2014': [('de', 'en'), ('cs', 'en'), ('fr', 'en'), ('ru', 'en'), ('hi', 'en')],
        'newsdev2015': [('fi', 'en'), ('en', 'fi')],
        'newstest2015': [('fi', 'en'), ('en', 'cs'), ('cs', 'en'), ('en', 'ru'), ('en', 'de'),
                         ('de', 'en'), ('ru', 'en'), ('en', 'fi')],
        'newsdev2016': [('en', 'ro'), ('ro', 'en'), ('tr', 'en'), ('en', 'tr')],
        'newstest2016': [('de', 'en'), ('en', 'de'), ('en', 'ro'), ('en', 'fi'), ('ro', 'en'),
                         ('ru', 'en'), ('fi', 'en'), ('en', 'ru'), ('tr', 'en'), ('cs', 'en'),
                         ('en', 'tr'), ('en', 'cs')],
        'newsdev2017': [('zh', 'en'), ('lv', 'en'), ('en', 'zh'), ('en', 'lv')],
        'newstest2017': [('zh', 'en'), ('ru', 'en'), ('en', 'fi'), ('lv', 'en'), ('en', 'de'),
                         ('de', 'en'), ('cs', 'en'), ('en', 'cs'), ('en', 'tr'), ('en', 'ru'),
                         ('tr', 'en'), ('fi', 'en'), ('en', 'zh'), ('en', 'lv')],
        'newsdev2018': [('et', 'en'), ('en', 'et')],
        'newstest2018': [('ru', 'en'), ('zh', 'en'), ('et', 'en'), ('en', 'fi'), ('en', 'de'),
                         ('de', 'en'), ('en', 'cs'), ('en', 'tr'), ('cs', 'en'), ('tr', 'en'),
                         ('en', 'ru'), ('en', 'et'), ('fi', 'en'), ('en', 'zh')],
        'newsdev2019': [('gu', 'en'), ('kk', 'en'), ('en', 'lt'), ('en', 'kk'), ('lt', 'en'),
                        ('en', 'gu')],
        'newstest2019': [('de', 'en'), ('de', 'fr'), ('kk', 'en'), ('en', 'de'), ('en', 'fi'),
                         ('ru', 'en'), ('zh', 'en'), ('gu', 'en'), ('en', 'kk'), ('en', 'zh'),
                         ('cs', 'de'), ('fi', 'en'), ('en', 'gu'), ('lt', 'en'), ('de', 'cs'),
                         ('en', 'lt'), ('en', 'ru'), ('en', 'cs'), ('fr', 'de')],
        'newsdev2020': [('iu', 'en'), ('en', 'ta'), ('ta', 'en'), ('pl', 'en'), ('en', 'iu'),
                        ('en', 'ja'), ('ja', 'en'), ('en', 'pl')]
    }
    for set_name, pairs in wmt_sets.items():
        for l1, l2 in pairs:
            src = f'dev/{set_name}-{l1}{l2}-src.{l1}.sgm'
            ref = f'dev/{set_name}-{l1}{l2}-ref.{l2}.sgm'
            name = f'{set_name}_{l1}{l2}'
            index.add_entry(Entry((l1, l2), name=name, filename='wmt20dev.tgz', in_paths=[src, ref],
                                  url='http://data.statmt.org/wmt20/translation-task/dev.tgz',
                                  cite=cite))
    # Multi parallel
    wmt_sets = {
        '2009': ['en', 'cs', 'de', 'es', 'fr'],
        '2010': ['en', 'cs', 'de', 'es', 'fr'],
        '2011': ['en', 'cs', 'de', 'es', 'fr'],
        '2012': ['en', 'cs', 'de', 'es', 'fr', 'ru'],
        '2013': ['en', 'cs', 'de', 'es', 'fr', 'ru'],
    }
    for year, langs in wmt_sets.items():
        for l1, l2 in itertools.combinations(langs, 2):
            name = f'newstest{year}'
            f1 = f'dev/{name}.{l1}'
            f2 = f'dev/{name}.{l2}'
            index.add_entry(Entry((l1, l2), name=name, filename='wmt20dev.tgz', in_paths=[f1, f2],
                                  in_ext='txt', cite=cite,
                                  url='http://data.statmt.org/wmt20/translation-task/dev.tgz'))

    for l1, l2 in [('ps', 'en'), ('km', 'en')]:
        for set_name in ['wikipedia.dev', 'wikipedia.devtest']:
            src = f'dev/{set_name}.{l1}-{l2}.{l1}'
            ref = f'dev/{set_name}.{l1}-{l2}.{l2}'
            name = f'{set_name.replace(".", "_")}_{l1}{l2}'
            index.add_entry(Entry((l1, l2), name=name, filename='wmt20dev.tgz', in_paths=[src, ref],
                                  url='http://data.statmt.org/wmt20/translation-task/dev.tgz',
                                  in_ext='txt', cite=cite))

    # ==== TED Talks 2.0 ar-en
    index.add_entry(Entry(('en', 'ar'), 'tedtalks_v2_clean', ext='tsv.xz',
                          url='http://data.statmt.org/ted-talks/en-ar.v2.aligned.clean.xz'))

    # ==== Europarl v10
    EP_v10 = "http://www.statmt.org/europarl/v10/training/europarl-v10.%s-%s.tsv.gz"
    wmt20_cite = None  # TODO: update
    for pair in ['cs en', 'cs pl', 'de en', 'de fr', 'es pt', 'fi en', 'fr en', 'lt en', 'pl en']:
        l1, l2 = pair.split()
        index.add_entry(
            Entry(langs=(l1, l2), name='europarl_v10', url=EP_v10 % (l1, l2), cite=wmt20_cite))

    # ==== PMIndia V1
    PMINDIA_v1 = "http://data.statmt.org/pmindia/v1/parallel/pmindia.v1.%s-%s.tsv"
    cite = r"""@ARTICLE{2020arXiv200109907H,
           author = {{Haddow}, Barry and {Kirefu}, Faheem},
            title = "{PMIndia -- A Collection of Parallel Corpora of Languages of India}",
          journal = {arXiv e-prints},
         keywords = {Computer Science - Computation and Language},
             year = "2020",
            month = "Jan",
              eid = {arXiv:2001.09907},
            pages = {arXiv:2001.09907},
    archivePrefix = {arXiv},
           eprint = {2001.09907}
    }"""
    for pair in ["as en", "bn en", "gu en", "hi en", "kn en", "ml en", "mni en", "mr en", "or en",
                 "pa en", "ta en", "te en", "ur en"]:
        l1, l2 = pair.split()
        # Note: listed as xx-en in URL but actually en-xx in the tsv; and its not compressed!
        index.add_entry(
            Entry(langs=(l2, l1), name='pmindia_v1', url=PMINDIA_v1 % (l1, l2), cite=cite))

    # Pashto - English  pseudo parallel dataset for alignment
    index.add_entry(Entry(langs=('en', 'ps'), name='wmt20_enps_aligntask',
                          url='http://data.statmt.org/wmt20/translation-task/ps-km/wmt20-sent.en-ps.xz',
                          cite=wmt20_cite, ext='tsv.xz'))

    # Pashto - English  mostly parallel dataset
    for name in ["GNOME.en-ps", "KDE4.en-ps", "Tatoeba.en-ps", "Ubuntu.en-ps", "bible.en-ps.clean",
                 "ted-wmt20.en-ps", "wikimedia.en-ps"]:
        ps = f'ps-parallel/{name}.ps'
        en = f'ps-parallel/{name}.en'
        url = 'http://data.statmt.org/wmt20/translation-task/ps-km/ps-parallel.tgz'
        name = name.replace('.en-ps', '').replace('.', '_').replace('-', '_').lower()
        entry = Entry(langs=('ps', 'en'), name=name, url=url, cite=wmt20_cite, in_paths=[ps, en],
                      filename='wmt20-psen-parallel.tgz', in_ext='txt')
        index.add_entry(entry)



#!/usr/bin/env python
#
# This file should contain popular papers and their experiment setup
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 4/23/20

from mtdata.entry import Experiment, Paper
from mtdata.index import Index


def load(index: Index):
    vaswani_etal_2017 = Paper(
        name="vaswani-etal-2017",
        title="Attention is all you need",
        url="https://papers.nips.cc/paper/7181-attention-is-all-you-need.pdf",
        cite="""@inproceedings{vaswani2017attention,
                  title={Attention is all you need},
                  author={Vaswani, Ashish and Shazeer, Noam and Parmar, Niki and Uszkoreit, Jakob and Jones, Llion and Gomez, Aidan N and Kaiser, {\L}ukasz and Polosukhin, Illia},
                  booktitle={Advances in neural information processing systems},
                  pages={5998--6008},
                  year={2017}
                }""",
        experiments=[
            Experiment.make(
                langs=('eng', 'deu'),
                train=['wmt13_europarl_v7', 'wmt13_commoncrawl', 'wmt18_news_commentary_v13'],
                tests=['newstest2013', 'newstest2014_deen'])
        ]
    )
    index.add_paper(vaswani_etal_2017)



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 4/8/20

from typing import Tuple, List, Optional, Set
from dataclasses import dataclass, field
from mtdata.parser import detect_extension
from mtdata.iso import iso3_code


class Entry:
    __slots__ = ('langs', 'name', 'url', 'filename', 'ext', 'in_paths', 'in_ext', 'cite', 'cols',
                 'is_archive')

    def __init__(self, langs: Tuple[str, str],
        name: str,
        url: str,
        filename: Optional[str] = None ,
        ext: Optional[str] = None,
        in_paths: Optional[List[str]] = None,
        in_ext: Optional[str] = None,
        cite: Optional[str] = None,
        cols: Optional[Tuple[int, int]] = None):

        assert isinstance(langs, tuple)
        assert len(langs) == 2
        for ch in '.-/* ':
            assert ch not in name, f"Character '{ch}' is not permitted in name {name}"

        self.langs = tuple(iso3_code(l, fail_error=True) for l in langs)
        self.name = name
        self.url = url
        self.filename = filename
        orig_name = self.url.split('/')[-1]
        self.ext = ext or detect_extension(filename or orig_name)
        self.filename = self.filename or f'{self.name}.{self.ext}'

        self.in_paths = in_paths
        self.in_ext = in_ext
        self.cite = cite
        self.cols = cols

        self.is_archive = self.ext in ('zip', 'tar', 'tar.gz', 'tgz')
        if self.is_archive:
            assert self.in_paths and len(self.in_paths) > 0, 'Archive entries must have in_paths'

    def is_swap(self, langs):
        return tuple(reversed(langs)) == tuple(self.langs)

    def __str__(self):
        return self.format(delim=' ')

    def format(self, delim: str = ' '):
        msg = f'{self.name}{delim}{"-".join(self.langs)}{delim}{self.url}{delim}' \
              f'{",".join(self.in_paths or [])}'
        return msg

    def is_noisy(self, seg1, seg2) -> bool:
        # None or Empty
        noisy = seg1 is None or seg2 is None or not seg1.strip() or not seg2.strip()
        return noisy


class JW300Entry(Entry):
    url: Tuple[str, str, str]  # (align.xml, src.xml, tgt.xml)


@dataclass
class Experiment:
    langs: Tuple[str, str]  # (lang1 , lang2)  lang1 -> lang2
    train: List[Entry]  # training should be merged from all these
    tests: List[Entry]  # multiple tests; one of them can be validation set
    papers: Set['Paper'] = field(default_factory=set)

    def __post_init__(self):
        self.langs = tuple(iso3_code(l, fail_error=True) for l in self.langs)
        for t in self.tests:
            assert t
        for t in self.train:
            assert t

    @classmethod
    def make(cls, langs: Tuple[str, str], train: List[str], tests: List[str]):
        from mtdata.index import INDEX
        train = [INDEX.get_entry(name, langs) for name in train]
        tests = [INDEX.get_entry(name, langs) for name in tests]
        return cls(langs, train=train, tests=tests)


@dataclass(eq=False)  # see for hash related issues: https://stackoverflow.com/a/52390734/1506477
class Paper:  # or Article

    name: str  # author1-etal-year
    title: str  # title
    url: str  # Paper url to be sure
    cite: str  # bibtex would be nice to display
    experiments: List[Experiment]

    langs: Set[Tuple[str, str]] = None

    def __post_init__(self):
        self.langs = self.langs or set(exp.langs for exp in self.experiments)
        for exp in self.experiments:
            exp.papers.add(self)



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 4/8/20

from mtdata.index import Entry, Index

def load(index: Index):
    # === Para crawl corpus
    PARACRAWL_v3 = 'https://s3.amazonaws.com/web-language-models/paracrawl/release3/%s-%s.bicleaner07.tmx.gz'
    cite = r"""@inproceedings{espla-etal-2019-paracrawl,
        title = "{P}ara{C}rawl: Web-scale parallel corpora for the languages of the {EU}",
        author = "Espl{\`a}, Miquel  and
          Forcada, Mikel  and
          Ram{\'\i}rez-S{\'a}nchez, Gema  and
          Hoang, Hieu",
        booktitle = "Proceedings of Machine Translation Summit XVII Volume 2: Translator, Project and User Tracks",
        month = aug,
        year = "2019",
        address = "Dublin, Ireland",
        publisher = "European Association for Machine Translation",
        url = "https://www.aclweb.org/anthology/W19-6721",
        pages = "118--119",
    }"""
    for pair in ['en cs', 'en de', 'en fi', 'en lt']:
        l1, l2 = pair.split()
        index.add_entry(
            Entry(langs=(l1, l2), name='paracrawl_v3', url=PARACRAWL_v3 % (l1, l2), cite=cite))

    # === Paracrawl V6
    PARACRAWL_v6 = 'https://s3.amazonaws.com/web-language-models/paracrawl/release6/%s-%s.txt.gz'
    for l2 in ['is', 'bg', 'hr', 'cs', 'da', 'nl', 'et', 'fi', 'fr', 'de', 'el', 'hu', 'ga', 'it', 'lv',
               'lt', 'mt', 'pl', 'pt', 'ro', 'sk', 'sl', 'es', 'sv']:
        l1 = 'en'
        index.add_entry(Entry(langs=(l1, l2), name='paracrawl_v6', url=PARACRAWL_v6 % (l1, l2),
                             cite=cite, ext='tsv.gz'))
    # these are bonus
    PARACRAWL_v6_B = 'https://s3.amazonaws.com/web-language-models/paracrawl/release6/%s-%s.bicleaner07.txt.gz'
    for l1, l2 in [('nl', 'fr'), ('pl', 'de')]:
        index.add_entry(Entry(langs=(l1, l2), name='paracrawl_v6', url=PARACRAWL_v6_B % (l1, l2),
                             cite=cite, ext='tsv.gz'))



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 4/4/20
from dataclasses import dataclass
from pathlib import Path
from mtdata.index import Entry
from mtdata import log
import wget
import portalocker
from hashlib import md5
from urllib.parse import urlparse
from .parser import detect_extension

MAX_TIMEOUT = 12 * 60 * 60  # 12 hours

@dataclass
class Cache:
    root: Path

    def __post_init__(self):
        if isinstance(self.root, str):
            self.root = Path(self.root)
        log.info(f"Local cache is at {self.root}")

    def get_entry(self, entry: Entry, fix_missing=True) -> Path:
        if entry.is_archive or entry.in_ext == 'opus_xces':
            local = self.get_local_in_paths(entry, fix_missing=fix_missing)
        else:
            local = self.get_local_path(entry.url, filename=entry.filename, fix_missing=fix_missing)
        return local

    def get_flag_file(self, file: Path):
        return file.with_name(file.name + '._valid')

    def get_local_path(self, url, filename=None, fix_missing=True):
        hostname = urlparse(url).hostname or 'nohost'
        filename = filename or url.split('/')[-1]
        assert hostname and filename
        mdf5_sum = md5(url.encode('utf-8')).hexdigest()
        local = self.root / hostname / mdf5_sum[:4] / mdf5_sum[4:] / filename
        if fix_missing:
            self.download(url, local)
        return local

    def get_local_in_paths(self, entry: Entry, fix_missing=True):
        if entry.in_ext == 'opus_xces': # this is special case
            l1_url, l2_url = entry.in_paths
            align_file = self.get_local_path(entry.url, fix_missing=fix_missing)
            l1_path = self.get_local_path(l1_url, fix_missing=True)
            l2_path = self.get_local_path(l2_url, fix_missing=True)
            #l1_dir = self._get_extracted_path(l1_url, ext='', fix_missing=fix_missing)
            #l2_dir = self._get_extracted_path(l2_url, ext='', fix_missing=fix_missing)
            #return [align_file, l1_dir, l2_dir]
            return [align_file, l1_path, l2_path]

        x_dir = self.get_extracted_path(entry, fix_missing=fix_missing)
        local_x_path = []
        for p in entry.in_paths:
            if '*' in p: # glob
                paths = list(x_dir.glob(p))
                if not paths:
                    raise Exception(f"{entry} with in path {p} did not find a match")
                local_x_path.extend(paths)
            else:
                local_x_path.append(x_dir / p)
        return local_x_path

    def get_extracted_path(self, entry: Entry, fix_missing=True):
        assert entry.is_archive
        return self._get_extracted_path(url=entry.url, ext=entry.ext, filename=entry.filename,
                                        fix_missing=fix_missing)

    def _get_extracted_path(self, url: str, ext=None, filename=None, fix_missing=True):
        ext = ext or detect_extension(url)
        # path of archive file, local
        local = self.get_local_path(url, filename=filename, fix_missing=fix_missing)
        # path of extract dir:  remove extension from dir name
        local_xdir = local.with_name(right_replace(local.name, f'.{ext}', ''))
        if fix_missing:
            if not local_xdir.exists() or not self.get_flag_file(local_xdir).exists():
                self.extract(local, ext, local_xdir)
                self.get_flag_file(local_xdir).touch()
        return local_xdir

    def download(self, url: str, save_at: Path):
        valid_flag = self.get_flag_file(save_at)
        lock_file = valid_flag.with_suffix("._lock")
        if valid_flag.exists() and save_at.exists():
            return save_at
        save_at.parent.mkdir(parents=True, exist_ok=True)

        log.info(f"Acquiring lock on {lock_file}\nif this gets stuck, delete the lock and restart")
        with portalocker.Lock(lock_file, 'w', timeout=MAX_TIMEOUT) as fh:
            # check if downloaded by  other parallel process
            if valid_flag.exists() and save_at.exists():
                return save_at
            log.info(f"Got the lock. Downloading {url} --> {save_at}")
            out_file = wget.download(url, out=str(save_at))
            log.info(" Done.")
            assert Path(out_file).resolve() == save_at.resolve()  # saved where we asked it to save

            valid_flag.touch()
            return save_at

    def extract(self, archive_file: Path, ext: str, x_dir: Path):
        assert archive_file.exists(), f'{archive_file} not found'
        valid_file = self.get_flag_file(x_dir)
        lock_file = valid_file.with_suffix('._lock')
        if x_dir.exists() and valid_file.exists():
            return  # already extracted

        x_dir.mkdir(parents=True, exist_ok=True)
        log.info(f"Acquiring lock on {lock_file}\nif this gets stuck, delete the lock and restart")
        with portalocker.Lock(lock_file, 'w', timeout=MAX_TIMEOUT) as fh:
            if valid_file.exists() and x_dir.exists():
                return
            if ext in {'tar', 'tgz', 'tar.gz', 'tar.bz2', 'tbz2', 'tar.xz', 'txz'}:
                log.info(f"Going to extract tar {archive_file} --> {x_dir}")
                import tarfile
                with tarfile.open(archive_file) as tar:
                    tar.extractall(path=x_dir)
            elif ext == 'zip':
                log.info(f"Going to extract zip {archive_file} --> {x_dir}")
                from zipfile import ZipFile
                with ZipFile(archive_file) as zip:
                    zip.extractall(path=x_dir)
            else:
                raise Exception(f'"{ext}" type extraction not supported')

            valid_file.touch()


def right_replace(string, old, new):
    """
    replaces last occurrence of old with new in a string
    :param string:
    :param old:
    :param new:
    :return:
    """
    return new.join(string.rsplit(old))


#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 4/8/20
from mtdata.entry import Entry
from mtdata.index import Index

def load(index: Index):
    CONTENT="""EESC2017::bg-de bg-en bg-fr cs-de cs-en cs-fr da-de da-en da-fr de-el de-en de-es de-et de-fi de-fr de-hr de-hu de-is de-it de-lt de-lv de-mt de-nl de-pl de-pt de-ro de-sk de-sl de-sv el-en el-fr en-es en-et en-fi en-fr en-hr en-hu en-is en-it en-lt en-lv en-mt en-nl en-pl en-pt en-ro en-sk en-sl en-sv es-fr et-fr fi-fr fr-hr fr-hu fr-is fr-it fr-lt fr-lv fr-mt fr-nl fr-pl fr-pt fr-ro fr-sk fr-sl fr-sv
    EMA2016::bg-de bg-en bg-fr cs-de cs-en cs-fr da-de da-en da-fr de-el de-en de-es de-et de-fi de-fr de-hr de-hu de-is de-it de-lt de-lv de-mt de-nl de-no de-pl de-pt de-ro de-sk de-sl de-sv el-en el-fr en-es en-et en-fi en-fr en-hr en-hu en-is en-it en-lt en-lv en-mt en-nl en-no en-pl en-pt en-ro en-sk en-sl en-sv es-fr et-fr fi-fr fr-hr fr-hu fr-is fr-it fr-lt fr-lv fr-mt fr-nl fr-no fr-pl fr-pt fr-ro fr-sk fr-sl fr-sv
    airbaltic::de-en de-et de-fi de-lt de-lv de-ru en-et en-fi en-lt en-lv en-ru et-fi et-lt et-lv et-ru fi-lt fi-lv fi-ru lt-lv lt-ru lv-ru
    czechtourism::de-en de-es de-fr de-it de-pl de-pt de-ru en-es en-fr en-it en-pl en-pt en-ru es-fr es-it es-pl es-pt es-ru fr-it fr-pl fr-pt fr-ru it-pl it-pt it-ru pl-pt pl-ru pt-ru
    ecb2017::bg-cs bg-da bg-de bg-el bg-en bg-es bg-et bg-fi bg-fr bg-hr bg-hu bg-it bg-lt bg-lv bg-mt bg-nl bg-pl bg-pt bg-ro bg-sk bg-sl bg-sv cs-da cs-de cs-el cs-en cs-es cs-et cs-fi cs-fr cs-hr cs-hu cs-it cs-lt cs-lv cs-mt cs-nl cs-pl cs-pt cs-ro cs-sk cs-sl cs-sv da-de da-el da-en da-es da-et da-fi da-fr da-hr da-hu da-it da-lt da-lv da-mt da-nl da-pl da-pt da-ro da-sk da-sl da-sv de-el de-en de-es de-et de-fi de-fr de-hr de-hu de-it de-lt de-lv de-mt de-nl de-pl de-pt de-ro de-sk de-sl de-sv el-en el-es el-et el-fi el-fr el-hr el-hu el-it el-lt el-lv el-mt el-nl el-pl el-pt el-ro el-sk el-sl el-sv en-es en-et en-fi en-fr en-hr en-hu en-it en-lt en-lv en-mt en-nl en-pl en-pt en-ro en-sk en-sl en-sv es-et es-fi es-fr es-hr es-hu es-it es-lt es-lv es-mt es-nl es-pl es-pt es-ro es-sk es-sl es-sv et-fi et-fr et-hr et-hu et-it et-lt et-lv et-mt et-nl et-pl et-pt et-ro et-sk et-sl et-sv fi-fr fi-hr fi-hu fi-it fi-lt fi-lv fi-mt fi-nl fi-pl fi-pt fi-ro fi-sk fi-sl fi-sv fr-hr fr-hu fr-it fr-lt fr-lv fr-mt fr-nl fr-pl fr-pt fr-ro fr-sk fr-sl fr-sv hr-hu hr-it hr-lt hr-lv hr-mt hr-nl hr-pl hr-pt hr-ro hr-sk hr-sl hr-sv hu-it hu-lt hu-lv hu-mt hu-nl hu-pl hu-pt hu-ro hu-sk hu-sl hu-sv it-lt it-lv it-mt it-nl it-pl it-pt it-ro it-sk it-sl it-sv lt-lv lt-mt lt-nl lt-pl lt-pt lt-ro lt-sk lt-sl lt-sv lv-mt lv-nl lv-pl lv-pt lv-ro lv-sk lv-sl lv-sv mt-nl mt-pl mt-pt mt-ro mt-sk mt-sl mt-sv nl-pl nl-pt nl-ro nl-sk nl-sl nl-sv pl-pt pl-ro pl-sk pl-sl pl-sv pt-ro pt-sk pt-sl pt-sv ro-sk ro-sl ro-sv sk-sl sk-sv sl-sv
    fold::en-lv
    rapid2016::bg-de bg-en bg-fr cs-de cs-fr da-de da-en da-fr de-el de-en de-es de-et de-fi de-fr de-hr de-hu de-is de-it de-lt de-lv de-mt de-nl de-no de-pl de-pt de-ro de-sk de-sl de-sv el-en el-fr en-es en-et en-fi en-fr en-hr en-hu en-is en-it en-lt en-lv en-mt en-nl en-no en-pt en-ro en-sk en-sl en-sv es-fr et-fr fi-fr fr-hr fr-hu fr-is fr-it fr-lt fr-lv fr-mt fr-nl fr-no fr-pl fr-pt fr-ro fr-sk fr-sl fr-sv
    rapid2019::cs-en de-en en-pl
    worldbank::en-es en-fr en-hr en-pl en-pt en-ro en-ru en-sq en-sr en-tr en-uk"""
    TILDE_CITE = """@inproceedings{rozis-skadins-2017-tilde,
    title = "Tilde {MODEL} - Multilingual Open Data for {EU} Languages",
    author = "Rozis, Roberts  and
      Skadi{\c{n}}{\v{s}}, Raivis",
    booktitle = "Proceedings of the 21st Nordic Conference on Computational Linguistics",
    month = may,
    year = "2017",
    address = "Gothenburg, Sweden",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/W17-0235",
    pages = "263--265",
}"""
    TILDE = 'https://tilde-model.s3-eu-west-1.amazonaws.com/%s.%s-%s.tmx.zip'
    for line in CONTENT.splitlines():
        line = line.strip()
        name, pairs = line.split('::')
        for pair in pairs.split(' '):
            l1, l2 = pair.split('-')
            url = TILDE % (name, l1, l2)
            index.add_entry(Entry(langs=(l1, l2), name=name, url=url, cite=TILDE_CITE,
                                    in_paths=["*.tmx"]))



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 5/8/20



def iso3_code(lang: str, fail_error=False, default=None) -> str:
    """
    Lookup 693_3 code of a language
    :param code: language name or a code, maybe of 2 or 3 letters
    :param default: return this value when no iso code is found
    :param fail_error: raise error when no ISO code is found
    :return: ISO 639_3 code
    """
    from mtdata.iso.iso639_3 import CODES as iso3_codes, name_to_code
    from mtdata.iso.iso639_2 import CODE2_TO_3, code2_to_code3_name

    if lang.lower() in iso3_codes:
        return lang.lower()
    if lang.lower() in CODE2_TO_3:
        _, name = code2_to_code3_name(lang.lower())
        iso3_code = name_to_code(name)
        if iso3_code:
            return iso3_code
    if name_to_code(lang, None):
        return name_to_code(lang)
    if fail_error:
        raise Exception(f"Unable to find ISO 639-3 code for '{lang}'. "
                        f"Please run\npython -m mtdata.iso | grep -i <name>\n"
                        f"to know the 3 letter ISO code for the language.")
    return default